package com.sun.jndi.ldap;

import javax.naming.directory.BasicAttribute;
import java.util.Vector;
import javax.naming.directory.Attribute;
import java.util.Iterator;
import com.sun.jndi.ldap.spi.LdapDnsProviderResult;
import javax.naming.directory.DirContext;
import java.util.Enumeration;
import javax.naming.StringRefAddr;
import javax.naming.RefAddr;
import javax.naming.NamingException;
import javax.naming.ldap.Control;
import javax.naming.ConfigurationException;
import javax.naming.Reference;
import com.sun.jndi.url.ldap.ldapURLContextFactory;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.Name;
import javax.naming.spi.InitialContextFactory;
import javax.naming.spi.ObjectFactory;

public final class LdapCtxFactory implements ObjectFactory, InitialContextFactory
{
    public static final String ADDRESS_TYPE = "URL";
    
    public LdapCtxFactory() {
    }
    
    @Override
    public Object getObjectInstance(Object o, Name name, Context context, Hashtable<?, ?> hashtable) throws Exception {
        if (!isLdapRef(o)) {
            return null;
        }
        return new ldapURLContextFactory().getObjectInstance(getURLs((Reference)o), name, context, hashtable);
    }
    
    @Override
    public Context getInitialContext(Hashtable<?, ?> hashtable) throws NamingException {
        try {
            String s = (hashtable != null) ? ((String)hashtable.get("java.naming.provider.url")) : null;
            if (s == null) {
                return new LdapCtx("", "localhost", 389, hashtable, false);
            }
            String[] fromList = LdapURL.fromList(s);
            if (fromList.length == 0) {
                throw new ConfigurationException("java.naming.provider.url property does not contain a URL");
            }
            return getLdapCtxInstance(fromList, hashtable);
        }
        catch (LdapReferralException ex) {
            if (hashtable != null && "throw".equals(hashtable.get("java.naming.referral"))) {
                throw ex;
            }
            return (Context)ex.getReferralContext(hashtable, (hashtable != null) ? ((Control[])(Control[])hashtable.get("java.naming.ldap.control.connect")) : null);
        }
    }
    
    private static boolean isLdapRef(Object o) {
        return o instanceof Reference && LdapCtxFactory.class.getName().equals(((Reference)o).getFactoryClassName());
    }
    
    private static String[] getURLs(Reference reference) throws NamingException {
        int n = 0;
        String[] array = new String[reference.size()];
        Enumeration<RefAddr> all = reference.getAll();
        while (all.hasMoreElements()) {
            RefAddr refAddr = (RefAddr)all.nextElement();
            if (refAddr instanceof StringRefAddr && refAddr.getType().equals("URL")) {
                array[n++] = (String)refAddr.getContent();
            }
            else {
                continue;
            }
        }
        if (n == 0) {
            throw new ConfigurationException("Reference contains no valid addresses");
        }
        if (n == reference.size()) {
            return array;
        }
        String[] array2 = new String[n];
        System.arraycopy(array, 0, array2, 0, n);
        return array2;
    }
    
    public static DirContext getLdapCtxInstance(Object o, Hashtable<?, ?> hashtable) throws NamingException {
        if ((o instanceof String)) {
            return getUsingURL((String)o, hashtable);
        }
        if ((o instanceof String[])) {
            return getUsingURLs((String[])(String[])o, hashtable);
        }
        throw new IllegalArgumentException("argument must be an LDAP URL String or array of them");
    }
    
    private static DirContext getUsingURL(String s, Hashtable<?, ?> hashtable) throws NamingException {
        try {
            LdapDnsProviderResult lookupEndpoints = LdapDnsProviderService.getInstance().lookupEndpoints(s, hashtable);
            NamingException ex = null;
            for (String s2 : lookupEndpoints.getEndpoints()) {
                try {
                    return getLdapCtxFromUrl(lookupEndpoints.getDomainName(), s, new LdapURL(s2), hashtable);
                }
                catch (NamingException ex2) {
                    ex = ex2;
                }
            }
            if (ex != null) {
                throw ex;
            }
            throw new NamingException("Could not resolve a valid ldap host");
        }
        catch (NamingException ex3) {
            throw ex3;
        }
        catch (Exception rootCause) {
            NamingException o = new NamingException();
            o.setRootCause(rootCause);
            throw o;
        }
    }
    
    private static LdapCtx getLdapCtxFromUrl(String domainName, String providerUrl, LdapURL ldapURL, Hashtable<?, ?> hashtable) throws NamingException {
        LdapCtx ldapCtx = new LdapCtx(ldapURL.getDN(), ldapURL.getHost(), ldapURL.getPort(), hashtable, ldapURL.useSsl());
        ldapCtx.setDomainName(domainName);
        ldapCtx.setProviderUrl(providerUrl);
        return ldapCtx;
    }
    
    private static DirContext getUsingURLs(String[] array, Hashtable<?, ?> hashtable) throws NamingException {
        NamingException ex = null;
        int length = array.length;
        int i = 0;
        while (i < length) {
            String s = array[i];
            try {
                return getUsingURL(s, hashtable);
            }
            catch (NamingException ex2) {
                ex = ex2;
                ++i;
            }
        }
        throw ex;
    }
    
    public static Attribute createTypeNameAttr(Class<?> clazz) {
        String[] typeNames = getTypeNames(clazz, new Vector(10));
        if (typeNames.length > 0) {
            BasicAttribute basicAttribute = new BasicAttribute(Obj.JAVA_ATTRIBUTES[6]);
            for (int i = 0; i < typeNames.length; ++i) {
                basicAttribute.add(typeNames[i]);
            }
            return basicAttribute;
        }
        return null;
    }
    
    private static String[] getTypeNames(Class<?> clazz, Vector<String> vector) {
        getClassesAux(clazz, vector);
        Class[] interfaces = clazz.getInterfaces();
        for (int i = 0; i < interfaces.length; ++i) {
            getClassesAux(interfaces[i], vector);
        }
        String[] array = new String[vector.size()];
        int n = 0;
        Iterator<Object> iterator = vector.iterator();
        while (iterator.hasNext()) {
            array[n++] = (String)iterator.next();
        }
        return array;
    }
    
    private static void getClassesAux(Class<?> clazz, Vector<String> vector) {
        if (!vector.contains(clazz.getName())) {
            vector.addElement(clazz.getName());
        }
        for (Class clazz2 = clazz.getSuperclass(); clazz2 != null; clazz2 = clazz2.getSuperclass()) {
            getTypeNames(clazz2, vector);
        }
    }
}
